///////////////////////////////////////////////////////////////////////////////
// Name:        wx/ptr_shrd.h
// Purpose:     compatibility wrapper for wx/sharedptr.h
// Author:      Vadim Zeitlin
// Created:     2009-02-03
// Copyright:   (c) 2009 Vadim Zeitlin <vadim@wxwidgets.org>
// Licence:     wxWindows licence
///////////////////////////////////////////////////////////////////////////////

// do not include this file in any new code, include wx/sharedptr.h instead
#include "wx/sharedptr.h"
