/////////////////////////////////////////////////////////////////////////////
// Name:        wx/helphtml.h
// Purpose:     Includes wx/html/helpctrl.h, for wxHtmlHelpController.
// Author:      Julian Smart
// Modified by:
// Created:     2003-05-24
// Copyright:   (c) Julian Smart
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef __WX_HELPHTML_H_
#define __WX_HELPHTML_H_

#if wxUSE_WXHTML_HELP
#include "wx/html/helpctrl.h"
#endif

#endif // __WX_HELPHTML_H_

